@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (c) 1998-2004 Michal Trojnara <Michal.Trojnara@mirt.net>
  *                 All Rights Reserved
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -31,163 +31,22 @@
 #include "common.h"
 #include "prototypes.h"
 
-#if defined(USE_UCONTEXT) || defined(USE_FORK)
-/* no need for critical sections */
-
-void enter_critical_section(SECTION_CODE i) {
-    /* empty */
-}
-
-void leave_critical_section(SECTION_CODE i) {
-    /* empty */
-}
-
-#endif /* USE_UCONTEXT || USE_FORK */
-
-#ifdef USE_UCONTEXT
-
-#if defined(CPU_SPARC) && ( \
-        defined(OS_SOLARIS2_0) || \
-        defined(OS_SOLARIS2_1) || \
-        defined(OS_SOLARIS2_2) || \
-        defined(OS_SOLARIS2_3) || \
-        defined(OS_SOLARIS2_4) || \
-        defined(OS_SOLARIS2_5) || \
-        defined(OS_SOLARIS2_6) || \
-        defined(OS_SOLARIS2_7) || \
-        defined(OS_SOLARIS2_8))
-#define ARGC 2
-#else
-#define ARGC 1
-#endif
-
-/* first context on the ready list is the active context */
-CONTEXT *ready_head=NULL, *ready_tail=NULL;         /* ready to execute */
-CONTEXT *waiting_head=NULL, *waiting_tail=NULL;     /* waiting on poll() */
-int next_id=1;
-
-unsigned long stunnel_process_id(void) {
-    return (unsigned long)getpid();
-}
-
-unsigned long stunnel_thread_id(void) {
-    return ready_head ? ready_head->id : 0;
-}
-
-static CONTEXT *new_context(void) {
-    CONTEXT *ctx;
-
-    /* allocate and fill the CONTEXT structure */
-    ctx=malloc(sizeof(CONTEXT));
-    if(!ctx) {
-        s_log(LOG_ERR, "Unable to allocate CONTEXT structure");
-        return NULL;
-    }
-    ctx->id=next_id++;
-    ctx->fds=NULL;
-    ctx->ready=0;
-    /* some manuals claim that initialization of ctx structure is required */
-    if(getcontext(&ctx->ctx)<0) {
-        free(ctx);
-        ioerror("getcontext");
-        return NULL;
-    }
-    ctx->ctx.uc_link=NULL; /* it should never happen */
-#if defined(__sgi) || ARGC==2 /* obsolete ss_sp semantics */
-    ctx->ctx.uc_stack.ss_sp=ctx->stack+STACK_SIZE-8;
-#else
-    ctx->ctx.uc_stack.ss_sp=ctx->stack;
-#endif
-    ctx->ctx.uc_stack.ss_size=STACK_SIZE;
-    ctx->ctx.uc_stack.ss_flags=0;
-
-    /* attach to the tail of the ready queue */
-    ctx->next=NULL;
-    if(ready_tail)
-        ready_tail->next=ctx;
-    ready_tail=ctx;
-    if(!ready_head)
-        ready_head=ctx;
-    return ctx;
-}
-
-/* s_log is not initialized here, but we can use log_raw */
-void sthreads_init(void) {
-    /* create the first (listening) context and put it in the running queue */
-    if(!new_context()) {
-        log_raw("Unable create the listening context");
-        exit(1);
-    }
-}
-
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
-    CONTEXT *ctx;
-
-    s_log(LOG_DEBUG, "Creating a new context");
-    ctx=new_context();
-    if(!ctx)
-        return -1;
-    s_log(LOG_DEBUG, "Context %ld created", ctx->id);
-    makecontext(&ctx->ctx, (void(*)(void))cli, ARGC, arg);
-    return 0;
-}
-
-#endif /* USE_UCONTEXT */
-
-#ifdef USE_FORK
-
-void sthreads_init(void) {
-    /* empty */
-}
-
-unsigned long stunnel_process_id(void) {
-    return (unsigned long)getpid();
-}
-
-unsigned long stunnel_thread_id(void) {
-    return 0L;
-}
-
-static void null_handler(int sig) {
-    signal(SIGCHLD, null_handler);
-}
-
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
-    switch(fork()) {
-    case -1:    /* error */
-        if(arg)
-            free(arg);
-        if(s>=0)
-            closesocket(s);
-        return -1;
-    case  0:    /* child */
-        if(ls>=0)
-            closesocket(ls);
-        signal(SIGCHLD, null_handler);
-        cli(arg);
-        exit(0);
-    default:    /* parent */
-        if(arg)
-            free(arg);
-        if(s>=0)
-            closesocket(s);
-    }
-    return 0;
-}
-
-#endif /* USE_FORK */
+#define STACK_SIZE 65536
 
 #ifdef USE_PTHREAD
 
+#include <pthread.h>
+
 static pthread_mutex_t stunnel_cs[CRIT_SECTIONS];
+
 static pthread_mutex_t lock_cs[CRYPTO_NUM_LOCKS];
 static pthread_attr_t pth_attr;
 
-void enter_critical_section(SECTION_CODE i) {
+void enter_critical_section(section_code i) {
     pthread_mutex_lock(stunnel_cs+i);
 }
 
-void leave_critical_section(SECTION_CODE i) {
+void leave_critical_section(section_code i) {
     pthread_mutex_unlock(stunnel_cs+i);
 }
 
@@ -257,43 +116,26 @@ int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
     return 0;
 }
 
-#endif /* USE_PTHREAD */
+#endif
 
 #ifdef USE_WIN32
 
-static CRITICAL_SECTION stunnel_cs[CRIT_SECTIONS];
-static CRITICAL_SECTION lock_cs[CRYPTO_NUM_LOCKS];
+CRITICAL_SECTION stunnel_cs[CRIT_SECTIONS];
 
-void enter_critical_section(SECTION_CODE i) {
+void enter_critical_section(section_code i) {
     EnterCriticalSection(stunnel_cs+i);
 }
 
-void leave_critical_section(SECTION_CODE i) {
+void leave_critical_section(section_code i) {
     LeaveCriticalSection(stunnel_cs+i);
 }
 
-static void locking_callback(int mode, int type,
-#ifdef HAVE_OPENSSL
-    const /* Callback definition has been changed in openssl 0.9.3 */
-#endif
-    char *file, int line) {
-    if(mode&CRYPTO_LOCK)
-        EnterCriticalSection(lock_cs+type);
-    else
-        LeaveCriticalSection(lock_cs+type);
-}
-
 void sthreads_init(void) {
     int i;
 
     /* Initialize stunnel critical sections */
     for(i=0; i<CRIT_SECTIONS; i++)
         InitializeCriticalSection(stunnel_cs+i);
-
-    /* Initialize OpenSSL locking callback */
-    for(i=0; i<CRYPTO_NUM_LOCKS; i++)
-        InitializeCriticalSection(lock_cs+i);
-    CRYPTO_set_locking_callback(locking_callback);
 }
 
 unsigned long stunnel_process_id(void) {
@@ -305,81 +147,101 @@ unsigned long stunnel_thread_id(void) {
 }
 
 int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
-    s_log(LOG_DEBUG, "Creating a new thread");
-    if(_beginthread((void(*)(void *))cli, STACK_SIZE, arg)==-1) {
-        ioerror("_beginthread");
+    DWORD iID;
+    HANDLE hThread;
+
+    log(LOG_DEBUG, "Creating a new thread");
+    hThread=CreateThread(NULL, STACK_SIZE,
+        (LPTHREAD_START_ROUTINE)cli, arg, 0, &iID);
+    if(!hThread) {
+        ioerror("CreateThread");
         return -1;
     }
-    s_log(LOG_DEBUG, "New thread created");
+    CloseHandle(hThread);
+    log(LOG_DEBUG, "New thread created");
     return 0;
 }
 
-#ifdef _WIN32_WCE
+#endif
 
-int _beginthread(void (*start_address)(void *),
-        int stack_size, void *arglist) {
-    DWORD thread_id;
-    HANDLE handle;
+#ifdef USE_FORK
 
-    handle=CreateThread(NULL, stack_size,
-        (LPTHREAD_START_ROUTINE)start_address, arglist, 0, &thread_id);
-    if(!handle)
+void enter_critical_section(section_code i) {
+    /* empty */
+}
+
+void leave_critical_section(section_code i) {
+    /* empty */
+}
+
+void sthreads_init(void) {
+    /* empty */
+}
+
+unsigned long stunnel_process_id(void) {
+    return (unsigned long)getpid();
+}
+
+unsigned long stunnel_thread_id(void) {
+    return 0L;
+}
+
+static void null_handler(int sig) {
+    signal(SIGCHLD, null_handler);
+}
+
+int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
+    switch(fork()) {
+    case -1:    /* error */
+        if(arg)
+            free(arg);
+        if(s>=0)
+            closesocket(s);
         return -1;
-    CloseHandle(handle);
+    case  0:    /* child */
+        if(ls>=0)
+            closesocket(ls);
+        signal(SIGCHLD, null_handler);
+        cli(arg);
+        exit(0);
+    default:    /* parent */
+        if(arg)
+            free(arg);
+        if(s>=0)
+            closesocket(s);
+    }
     return 0;
 }
 
-void _endthread(void) {
-    ExitThread(0);
-}
-
-#endif /* !defined(_WIN32_WCE) */
-
-#endif /* USE_WIN32 */
+#endif
 
 #ifdef DEBUG_STACK_SIZE
 
-#define STACK_RESERVE (STACK_SIZE/8)
-#define VERIFY_AREA ((STACK_SIZE-STACK_RESERVE)/sizeof(u32))
-#define TEST_VALUE 0xdeadbeef
+#define STACK_RESERVE (STACK_SIZE/2)
+#define TEST_VALUE 44
+
+/* Some heuristic to determine the usage of client stack size.  It can
+ * fail on some platforms and/or OSes, so it'is not enabled by default. */
 
-/* some heuristic to determine the usage of client stack size */
 void stack_info(int init) { /* 1-initialize, 0-display */
-    u32 table[VERIFY_AREA];
-    int i, num;
-    static int min_num=VERIFY_AREA;
+    char table[STACK_SIZE-STACK_RESERVE];
+    int i;
 
     if(init) {
-        for(i=0; i<VERIFY_AREA; i++)
-            table[i]=TEST_VALUE;
+        memset(table, TEST_VALUE, STACK_SIZE-STACK_RESERVE);
     } else {
-        /* the stack is growing down */
-        for(i=0; i<VERIFY_AREA; i++)
-            if(table[i]!=TEST_VALUE)
-                break;
-        num=i;
-        /* the stack is growing up */
-        for(i=0; i<VERIFY_AREA; i++)
-            if(table[VERIFY_AREA-i-1]!=TEST_VALUE)
-                break;
-        if(i>num) /* use the higher value */
-            num=i;
-        if(num<64) {
-            s_log(LOG_NOTICE, "STACK_RESERVE is too high");
-            return;
-        }
-        if(num<min_num)
-            min_num=num;
-        s_log(LOG_NOTICE,
-            "stack_info: size=%d, current=%d (%d%%), maximum=%d (%d%%)",
-            STACK_SIZE,
-            (int)((VERIFY_AREA-num)*sizeof(u32)),
-            (int)((VERIFY_AREA-num)*sizeof(u32)*100/STACK_SIZE),
-            (int)((VERIFY_AREA-min_num)*sizeof(u32)),
-            (int)((VERIFY_AREA-min_num)*sizeof(u32)*100/STACK_SIZE));
+        i=0;
+        while(i<STACK_SIZE-STACK_RESERVE && table[i]==TEST_VALUE)
+            i++;
+        if(i<64)
+            log(LOG_ERR, "STACK_RESERVE is to high");
+        else
+            log(LOG_NOTICE, "stack_info: %d of %d bytes used (%d%%)",
+                STACK_SIZE-STACK_RESERVE-i, STACK_SIZE,
+                (STACK_SIZE-STACK_RESERVE-i)*100/STACK_SIZE);
     }
 }
 
-#endif /* DEBUG_STACK_SIZE */
+#endif DEBUG_STACK_SIZE
 
 /* End of sthreads.c */
